@@ -3,6 +3,7 @@ Pypodo Tests
 """
 
 import re
+import os
 import sys
 import unittest
 from io import StringIO
@@ -30,6 +31,7 @@ from pypodo.__pypodo__ import (
     tag,
     test_date,
     untag,
+    generate_key_fernet,
 )
 
 
@@ -59,6 +61,48 @@ class TestMethodsEntryPoint(unittest.TestCase):
     Class Test for entrypoints
     """
 
+    @patch("os.path.isfile")
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="[SYSTEM]\nmessagelevel = debug",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_pypodo_entry_point_crypt(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        Test pypodo entry point for crypt
+        """
+        with patch.object(sys, "argv", [pypodo, "crypt"]):
+            mock_isfile.return_value = True
+            pypodo(mock_open_file)
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "error   : not crypt key",
+            )
+
+    @patch("os.path.isfile")
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="[SYSTEM]\nmessagelevel = debug",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_pypodo_entry_point_decrypt(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        Test pypodo entry point for crypt
+        """
+        with patch.object(sys, "argv", [pypodo, "decrypt"]):
+            mock_isfile.return_value = True
+            pypodo(mock_open_file)
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "error   : not decrypt key",
+            )
+
     @patch("os.path.isfile")
     @patch("builtins.open", new_callable=mock_open)
     @patch("sys.stdout", new_callable=StringIO)
@@ -468,14 +512,47 @@ class TestMethodsErrors(unittest.TestCase):
         """
         with patch.object(sys, "argv", [pypodo, listtask]):
             mock_isfile.return_value = True
-            listtask(mock_open_file)
-            self.assertEqual(
-                escape_ansi(mock_print.getvalue().rstrip("\n")),
-                "warning : this line has not a valid format in .todo"
-                " - 1 my task #te#st\nwarning : this line has not a"
-                " valid format in .todo - a other task\nerror   :"
-                " verify the .todo file",
-            )
+            try:
+                error = False
+                listtask(mock_open_file)
+            except SystemExit:
+                self.assertEqual(
+                    escape_ansi(mock_print.getvalue().rstrip("\n")),
+                    "warning : this line has not a valid format in .todo"
+                    " - 1 my task #te#st\nwarning : this line has not a"
+                    " valid format in .todo - a other task\nerror   :"
+                    " verify the .todo file. Is it encrypted?",
+                )
+                error = True
+            self.assertEqual(error, True)
+
+    @patch("os.path.isfile")
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="1 my task te#st\na other task",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_sort_with_other_invalid_todo_return_warning_and_error(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        test_sort_with_invalid_todo_return_warning_and_error
+        """
+        with patch.object(sys, "argv", [pypodo, listtask]):
+            mock_isfile.return_value = True
+            try:
+                error = False
+                listtask(mock_open_file)
+            except SystemExit:
+                self.assertEqual(
+                    escape_ansi(mock_print.getvalue().rstrip("\n")),
+                    "warning : this line has not a"
+                    " valid format in .todo - a other task\nerror   :"
+                    " verify the .todo file. Is it encrypted?",
+                )
+                error = True
+            self.assertEqual(error, True)
 
 
 class TestMethodsWarnings(unittest.TestCase):
@@ -483,6 +560,84 @@ class TestMethodsWarnings(unittest.TestCase):
     Class Test for Warnings
     """
 
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="[SYSTEM]\nmessagelevel = debug\ntodofile = /tmp/.todo",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_pypodo_crypt_notodofile(
+        self, mock_print, mock_open_file
+    ):
+        """
+        Test pypodo crypt with not todofile
+        """
+        with patch.object(sys, "argv", [pypodo, "crypt", "password"]):
+            if os.path.isfile("/tmp/.todo"):
+                os.remove("/tmp/.todo")
+            pypodo()
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "debug   : fernet password - password"
+                "\ndebug   : fernet key - NWY0ZGNjM2I1YWE3NjVkNjFkODMyN2RlYjg4MmNmOTk="
+                "\nerror   : not .todo file",
+            )
+
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="[SYSTEM]\nmessagelevel = debug\ntodofile = /tmp/.todo\nkey = password",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_pypodo_list_with_key_and_nottodofile(
+        self, mock_print, mock_open_file
+    ):
+        """
+        Test pypodo crypt with not todofile
+        """
+        with patch.object(sys, "argv", [pypodo, "list"]):
+            if os.path.isfile("/tmp/.todo"):
+                os.remove("/tmp/.todo")
+            try:
+                error = False
+                pypodo()
+            except TypeError:
+                self.assertEqual(
+                    escape_ansi(mock_print.getvalue().rstrip("\n")),
+                    "debug   : fernet password - password"
+                    "\ndebug   : fernet key - NWY0ZGNjM2I1YWE3NjVkNjFkODMyN2RlYjg4MmNmOTk="
+                    "\ndebug   : creating .todolist file\ninfo    : creating .todolist file"
+                    "\nwarning : the todolist is empty\ndebug   : fernet password - password"
+                    "\ndebug   : fernet key - NWY0ZGNjM2I1YWE3NjVkNjFkODMyN2RlYjg4MmNmOTk=",
+                )
+                error = True
+            self.assertEqual(error, True)
+
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="[SYSTEM]\nmessagelevel = debug\ntodofile = /tmp/.todo",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_pypodo_decrypt_notodofile(
+        self, mock_print, mock_open_file
+    ):
+        """
+        Test pypodo crypt with not todofile
+        """
+        with patch.object(
+            sys, "argv", [pypodo, "decrypt", "password"]
+        ):
+            if os.path.isfile("/tmp/.todo"):
+                os.remove("/tmp/.todo")
+            pypodo()
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "debug   : fernet password - password"
+                "\ndebug   : fernet key - NWY0ZGNjM2I1YWE3NjVkNjFkODMyN2RlYjg4MmNmOTk="
+                "\nerror   : not .todo file",
+            )
+
     @patch("os.path.isfile")
     @patch("builtins.open", new_callable=mock_open)
     @patch("sys.stdout", new_callable=StringIO)
@@ -708,6 +863,59 @@ class TestMethodsWarnings(unittest.TestCase):
                 "warning : the task has not a valid format - #task",
             )
 
+    @patch("os.path.isfile")
+    @patch("builtins.open", new_callable=mock_open)
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_add_other_task_not_valid_format_return_warning(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        test_add_task_not_valid_format_return_warning
+        """
+        with patch.object(sys, "argv", [pypodo, add, "#task #task"]):
+            mock_isfile.return_value = True
+            add(mock_open_file)
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "warning : the task has not a valid format - #task #task",
+            )
+
+    @patch("os.path.isfile")
+    @patch("builtins.open", new_callable=mock_open)
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_add_third_task_not_valid_format_return_warning(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        test_add_task_not_valid_format_return_warning
+        """
+        with patch.object(sys, "argv", [pypodo, add, "task # #task"]):
+            mock_isfile.return_value = True
+            add(mock_open_file)
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "warning : the task has not a valid format - task # #task",
+            )
+
+    @patch("os.path.isfile")
+    @patch("builtins.open", new_callable=mock_open)
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_add_third_task_with_valid_format_return_not_warning(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        test_add_task_not_valid_format_return_warning
+        """
+        with patch.object(
+            sys, "argv", [pypodo, add, "task#task #task"]
+        ):
+            mock_isfile.return_value = True
+            add(mock_open_file)
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "info    : task is added to the todolist - 1 task#task #task",
+            )
+
     @patch("os.path.isfile")
     @patch("builtins.open", new_callable=mock_open)
     @patch("sys.stdout", new_callable=StringIO)
@@ -949,7 +1157,7 @@ class TestMethodsOthers(unittest.TestCase):
     @patch(
         "builtins.open",
         new_callable=mock_open,
-        read_data="1 task #tag1 #tag2",
+        read_data="1 t#urgentask# #tag1 #tag2",
     )
     @patch("sys.stdout", new_callable=StringIO)
     def test_list_print_one_task_and_tags(
@@ -963,7 +1171,7 @@ class TestMethodsOthers(unittest.TestCase):
             listtask(mock_open_file)
             self.assertEqual(
                 escape_ansi(mock_print.getvalue().rstrip("\n")),
-                "1 task #tag1 #tag2",
+                "1 t#urgentask# #tag1 #tag2",
             )
 
     @freeze_time("2020-10-14")
@@ -992,7 +1200,7 @@ class TestMethodsOthers(unittest.TestCase):
     @patch(
         "builtins.open",
         new_callable=mock_open,
-        read_data="1 task1 #tag\n2 task2 #tag2",
+        read_data="1 task1 #tag\n2 task2 #tag2\n3 task3#tag #tag3",
     )
     @patch("sys.stdout", new_callable=StringIO)
     def test_list_with_filter_return_result(
@@ -1148,6 +1356,31 @@ class TestMethodsOthers(unittest.TestCase):
                 " - 2 task2 #tag #tag3 -> 2 task2 #tag3",
             )
 
+    @patch("os.path.isfile")
+    @patch(
+        "builtins.open",
+        new_callable=mock_open,
+        read_data="1 task#tag #tag2\n2 task#tag2 #tag #tag3",
+    )
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_untag_other_one_task(
+        self, mock_print, mock_open_file, mock_isfile
+    ):
+        """
+        test_untag_one_task
+        """
+        with patch.object(sys, "argv", [pypodo, untag, "tag", "2"]):
+            mock_isfile.return_value = True
+            untag(mock_open_file)
+            mock_open_file().write.assert_called_with(
+                "2 task#tag2 #tag3\n"
+            )
+            self.assertEqual(
+                escape_ansi(mock_print.getvalue().rstrip("\n")),
+                "info    : tag deleted from the task of the todolist"
+                " - 2 task#tag2 #tag #tag3 -> 2 task#tag2 #tag3",
+            )
+
     @patch("os.path.isfile")
     @patch(
         "builtins.open",
@@ -1260,10 +1493,10 @@ class TestMethodsOthers(unittest.TestCase):
     @patch(
         "builtins.open",
         new_callable=mock_open,
-        read_data="1 task #tag\n2 task2 #tag\n3 task3 #tag2\n4 task4",
+        read_data="1 ta#urgentsk #tag\n2 task2 #tag\n3 task3 #tag2\n4 task4",
     )
     @patch("sys.stdout", new_callable=StringIO)
-    def test_listtag_return_result(
+    def test_listtag_return_other_result(
         self, mock_print, mock_open_file, mock_isfile
     ):
         """
@@ -1369,6 +1602,15 @@ class TestMethodsOthers(unittest.TestCase):
         helppypodo()
         mock_print.assert_called_with(AnyStringWith("SYNOPSIS"))
 
+    def test_generate_key_fernet(self):
+        """
+        test key generators
+        """
+        self.assertEqual(
+            generate_key_fernet("password"),
+            "NWY0ZGNjM2I1YWE3NjVkNjFkODMyN2RlYjg4MmNmOTk=",
+        )
+
 
 def escape_ansi(line):
     """
